


bl_info = {
    "name": "BlendRef",
    "author": "Pictofilmo",
    "version": (1, 7,0),
    "blender": (3, 6, 0),
    "location": "View3D > BlendRef",
    "description": "Reference Images and videos manager",
    "warning": "",
    "doc_url": "",
    "category": "3Dview",
}



import bpy
import os
from bpy.ops import colorpicker
import json
import mathutils
from bpy_extras.io_utils import ImportHelper
from bpy.types import (Panel,
                       Menu,
                       Operator,
                       PropertyGroup,
                       )
from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,
                       )
# ------------------------------------------------------------------------
#    Scene Properties
# ------------------------------------------------------------------------

class BLENDREF_prop(PropertyGroup):
    xloc: IntProperty(
        name = "X",
        description = "X location on import",
        default = 5000,
        min = 1,
        max = 10000
        )
    yloc: IntProperty(
        name = "Y",
        description = "Y location on import",
        default = 5000,
        min = 1,
        max = 10000
        )
    num: IntProperty(
        name = "Arrange in columns",
        description = "number of images columns",
        default = 5,
        min = 1,
        max = 500
        )
    my_col : FloatVectorProperty(
        name= "背景颜色", 
        subtype= 'COLOR' , 
        default= [0.01,0.01,0.01],
        min = 0,
        max = 1
        )
    path : StringProperty(
        name="目录路径",
        description="",
        default="",
        maxlen=1024,
        subtype='DIR_PATH')
    path2 : StringProperty(
        name="",
        description="",
        default="",
        maxlen=1024,
        subtype='DIR_PATH')

    frame1: IntProperty(
        name = "偏移量",
        description = "要应用于所选影片的帧偏移值",
        default = 0,
        min = 0,
        max = 1000000
        )

    zoom: FloatProperty(
        name = "缩放",
        description = "",
        default = 3,
        min = 0.1,
        max = 50
        )
    rezx: IntProperty(
        name = "分辨率 X",
        description = "",
        default = 1000,
        min = 10,
        max = 100000
        )
    rezy: IntProperty(
        name = "分辨率 Y",
        description = "",
        default = 1000,
        min = 10,
        max = 100000
        )
    percent: IntProperty(
        name = "%",
        description = "",
        default = 100,
        min = 10,
        max = 1000
        )
# ------------------------------------------------------------------------
#    FUNCTIONS
# ------------------------------------------------------------------------    

def in_BREF_coll():
    # return true if active object is in one BlendRef collection
    list=[]
    for myCol in bpy.data.collections:
        if myCol.name == "BlendRef-Images":        
            i_collection = bpy.data.collections["BlendRef-Images"]
            i_objects = i_collection.objects
            for o in i_objects:
                list.append(o)
            break    
    for myCol in bpy.data.collections:
        if myCol.name == "BlendRef-Movies":     
            v_collection = bpy.data.collections["BlendRef-Movies"]
            v_objects = v_collection.objects
            for o in v_objects:
                list.append(o)
            break               
    for ob in list:
        if ob==bpy.context.view_layer.objects.active:  
            return True            
            break

def MODE_switch():
    if bpy.context.view_layer.objects.active==None:
        print('NO')
        pass
    else:
        startmode = bpy.context.object.mode     
        if startmode=='POSE':
            bpy.ops.object.posemode_toggle()
        if startmode=='EDIT':
            bpy.ops.object.editmode_toggle()
        if startmode=='SCULPT':
            bpy.ops.sculpt.sculptmode_toggle()
        if startmode=='VERTEX_PAINT':
            bpy.ops.paint.vertex_paint_toggle()            
        if startmode=='WEIGHT_PAINT':
            bpy.ops.paint.weight_paint_toggle()
        if startmode=='TEXTURE_PAINT':
            bpy.ops.paint.texture_paint_toggle()  
    return    

# ------------------------------------------------------------------------
#    Operators
# ------------------------------------------------------------------------

class TEST_button(bpy.types.Operator):
    bl_idname = "wm.testbutton"
    bl_label = "DEBUG"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        current_mode = bpy.context.object.mode
        print(current_mode)     
        MODE_switch()                
        return{'FINISHED'} 
    
#        
#    def invoke(self, context, event):
#        return context.window_manager.invoke_props_dialog(self,width = 300) 
#    def draw(self, context):
##------------- VIEWPORT MODE        
#        if bpy.context.object == None:
#            if bpy.data.objects != None:
#                for ob in bpy.data.objects:
#                    bpy.context.view_layer.objects.active = ob
#                    startmode = bpy.context.object.mode
#            else:
#                startmode = 'OBJECT'
#        else:
#            startmode = bpy.context.object.mode
#        
#        self.layout.label(text="Viewport is in "+startmode+" mode",icon='WORKSPACE')
#        
##-------------MOVIE INFOS

#        if "BlendRef-Movies" in bpy.data.collections:
#            collection = bpy.data.collections["BlendRef-Movies"]
#            objects = collection.objects
#            if len(bpy.context.selected_objects) >0:
#                ob=bpy.context.object                
#                if ob.name in objects:
#                    filepath = ob.data.filepath
#                    name= ob.data.name
#                    frame_duration=bpy.data.images[name].frame_duration
#                    sizeX=bpy.data.images[name].size[0]
#                    sizeY=bpy.data.images[name].size[1]
#                    frame_current=bpy.data.objects[ob.name].image_user.frame_current
#                    print("number of frames:",frame_duration)
#                    print("size:",sizeX,"x",sizeY)
#                    print("Current frame:",frame_current)  
#                    
#     
#                    self.layout.label(text= "number of frames: "+str(frame_duration))
#                    self.layout.label(text= "Size: "+str(sizeX)+"x"+str(sizeY))
#                    self.layout.label(text= "Current frame: "+str(frame_current))
#        
##-------------IMAGE INFOS

#        if "BlendRef-Images" in bpy.data.collections:
#            collection = bpy.data.collections["BlendRef-Images"]
#            objects = collection.objects
#            if len(bpy.context.selected_objects) >0:
#                ob=bpy.context.object                
#                if ob.name in objects:
#                    filepath = ob.data.filepath
#                    name= ob.data.name
#                    file_format=bpy.data.images[name].file_format
#                    sizeX=bpy.data.images[name].size[0]
#                    sizeY=bpy.data.images[name].size[1]
#                    print("file format:", file_format)
#                    print("size:",sizeX,"x",sizeY)
#                    
#                    self.layout.label(text= "File format: "+str(file_format))
#                    self.layout.label(text= "Size: "+str(sizeX)+"x"+str(sizeY))
#      

    
class APPEND_button(bpy.types.Operator):
    bl_idname = "wm.importfolder"
    bl_label = "从文件夹导入"
    bl_description="从文件夹导入所有图像"
    def execute(self, context):
        MODE_switch()
        imgpath= bpy.context.scene.bref_prop.path        
        if os.path.exists(imgpath):

            sel=bpy.context.view_layer.objects.active
            sels = bpy.context.selected_objects
            
            files= os.listdir(imgpath)
            folder = os.path.dirname(imgpath)
            data = bpy.context.scene.collection
            colname= "BlendRef-Images"
            origin_collection= bpy.context.view_layer.active_layer_collection
            if colname in bpy.data.collections:
                bref_collection = bpy.data.collections.get(colname)
            else:
                bref_collection = bpy.data.collections.new(colname)
                data.children.link(bref_collection)
                
            layer_collection = bpy.context.view_layer.layer_collection.children[colname]
            bpy.context.view_layer.active_layer_collection = layer_collection              
              
            col=bpy.context.scene.bref_prop.num
            xl=bpy.context.scene.bref_prop.xloc
            yl=bpy.context.scene.bref_prop.yloc
            index=-1
            y=0
            for x in files:
                if x.endswith(".jpg") or x.endswith(".jpeg") or x.endswith(".png") or x.endswith(".JPG") or x.endswith(".JPEG") or x.endswith(".PNG"):
                    if x not in bpy.data.images:
                        img = bpy.data.images.load(imgpath+x)
                    else:
                        img= bpy.data.images[x]
                        #print(x,"is recycled")
                    index+=1.1
                    if index >col:
                        y-=1
                        index=0.1
                    name="BlendRef_"+x
                    if name not in bpy.data.objects:  
                        print("empty dos not exist")          
                        bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(index+xl, y+yl, 0), scale=(1, 1, 1))            
                        EMPT=bpy.context.view_layer.objects.active
                        EMPT.empty_display_type = 'IMAGE'            
                        EMPT.data=img            
                        EMPT.name="BlendRef_"+x
                    else:
                        pass
                        #print("empty exists")
                                
           # bpy.context.view_layer.active_layer_collection=origin_collection
            bpy.context.space_data.shading.background_type = 'VIEWPORT'
            my_col= bpy.context.scene.bref_prop.my_col
            bpy.context.space_data.shading.background_color = my_col
            bpy.context.space_data.overlay.show_ortho_grid = False
            bpy.context.space_data.overlay.show_floor = False
            bpy.context.space_data.overlay.show_axis_x = False
            bpy.context.space_data.overlay.show_axis_y = False
            try:
                bpy.ops.view3d.view_axis(type="TOP", align_active=True)
            except:
                pass
            bpy.ops.view3d.view_selected(use_all_regions=False)
            collection = bpy.data.collections["BlendRef-Images"]
            objects = collection.objects
            for ob in objects:
                ob.select_set(True)
                bpy.context.view_layer.objects.active = ob 
            #bpy.ops.view3d.view_axis(type="TOP", align_active=True)
            bpy.ops.view3d.view_selected(use_all_regions=False)
            
            bpy.ops.object.select_all(action='DESELECT')
            bpy.context.view_layer.objects.active = sel
            for o in sels:
                o.select_set(True)
            
                
            bpy.context.space_data.show_gizmo_empty_image = False    
            #lock VIEW3D    
            context = bpy.context
            areatype = context.area.type
            #print(areatype)
            if areatype=="VIEW_3D":
                bpy.context.area.spaces[0].region_3d.lock_rotation = True
            bpy.context.view_layer.active_layer_collection=origin_collection     

        else:
            self.report({'ERROR'}, "请设置有效的绝对路径")
            
        bpy.context.space_data.overlay.show_overlays = True  
        return {'FINISHED'}


class ImportImageOperator(bpy.types.Operator, ImportHelper):
    """Import a image file"""
    bl_idname = "wm.import_image"
    bl_label = "导入图像"
    bl_options = {'REGISTER', 'UNDO'}
    filename_ext = ".jpg,.jpeg,.JPEG,.JPG,.png,.PNG"
    filter_glob: bpy.props.StringProperty(default="*.jpg;*.jpeg;*.JPEG;*.JPG;*.png;*.PNG",options={'HIDDEN'})
    def execute(self, context):
        MODE_switch()
            
        filepath = self.filepath
        print(filepath)
        x = os.path.basename(filepath)
        print(x)

        sel=bpy.context.view_layer.objects.active
        sels = bpy.context.selected_objects
        #folder = os.path.dirname(imgpath)
        data = bpy.context.scene.collection
        colname= "BlendRef-Images"
        origin_collection= bpy.context.view_layer.active_layer_collection
        col=bpy.context.scene.bref_prop.num
        xl=bpy.context.scene.bref_prop.xloc
        yl=bpy.context.scene.bref_prop.yloc       
        if colname in bpy.data.collections:
            list=[]
            d=()
            bref_collection = bpy.data.collections.get(colname)
            objects = bref_collection.objects
            for ob in objects:
                if ob.type == 'EMPTY' and ob.empty_display_type == 'IMAGE':
                    list.append(ob)
                else:
                    bref_collection.objects.unlink(ob)
                    bpy.context.scene.collection.objects.link(ob)
            if len(list)>0:
                min_x_object = min(list, key=lambda obj: obj.location.x)
                x_min=min_x_object.location.x-xl
            else:
                x_min= 0
                y_max= 0            

        else:
            bref_collection = bpy.data.collections.new(colname)
            data.children.link(bref_collection)
            x_min= 0
            y_max= 0
            
        layer_collection = bpy.context.view_layer.layer_collection.children[colname]
        bpy.context.view_layer.active_layer_collection = layer_collection              
          

        if x not in bpy.data.images:
            img = bpy.data.images.load(filepath)
        else:
            img= bpy.data.images[x]
            print(x,"is recycled")
        index=-1.1
        name="BlendRef_"+x
        if name not in bpy.data.objects:  
            print("empty dos not exist")          
            bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(index+x_min+xl, yl, 0), scale=(1, 1, 1))            
            EMPT=bpy.context.view_layer.objects.active
            EMPT.empty_display_type = 'IMAGE'  
            EMPT.empty_display_size = 1         
            EMPT.data=img           
            EMPT.name="BlendRef_"+x


        else:
            print("empty exists")
        bpy.context.space_data.shading.background_type = 'VIEWPORT'
        my_col= bpy.context.scene.bref_prop.my_col
        bpy.context.space_data.shading.background_color = my_col
        bpy.context.space_data.overlay.show_ortho_grid = False
        bpy.context.space_data.overlay.show_floor = False
        bpy.context.space_data.overlay.show_axis_x = False
        bpy.context.space_data.overlay.show_axis_y = False
        try:
            bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        except:
            pass
        bpy.ops.view3d.view_selected(use_all_regions=False)
        collection = bpy.data.collections["BlendRef-Images"]
        objects = collection.objects
        for ob in objects:
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob 
        #bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        bpy.ops.view3d.view_selected(use_all_regions=False)
        
        bpy.ops.object.select_all(action='DESELECT')
        bpy.context.view_layer.objects.active = sel
        for o in sels:
            o.select_set(True)
        
        bpy.context.space_data.show_gizmo_empty_image = False
        #lock VIEW3D    
        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = True

        bpy.context.view_layer.active_layer_collection=origin_collection

        bpy.context.space_data.overlay.show_overlays = True
        return{'FINISHED'}  
        







class ImportImageSeqOperator(bpy.types.Operator, ImportHelper):
    """Import a image file"""
    bl_idname = "wm.import_image_seq"
    bl_label = "导入图像序列"
    bl_options = {'REGISTER', 'UNDO'}
    filename_ext = ".jpg,.jpeg,.JPEG,.JPG,.png,.PNG"
    filter_glob: bpy.props.StringProperty(default="*.jpg;*.jpeg;*.JPEG;*.JPG;*.png;*.PNG",options={'HIDDEN'})
    def execute(self, context):
        MODE_switch()
            
        filepath = self.filepath
        print(filepath)
        x = os.path.basename(filepath)
        print(x)

        sel=bpy.context.view_layer.objects.active
        sels = bpy.context.selected_objects
        #folder = os.path.dirname(imgpath)
        data = bpy.context.scene.collection
        colname= "BlendRef-Movies"
        origin_collection= bpy.context.view_layer.active_layer_collection
        col=bpy.context.scene.bref_prop.num
        xl=bpy.context.scene.bref_prop.xloc
        yl=bpy.context.scene.bref_prop.yloc       
        if colname in bpy.data.collections:
            list=[]
            d=()
            bref_collection = bpy.data.collections.get(colname)
            objects = bref_collection.objects
            for ob in objects:
                if ob.type == 'EMPTY' and ob.empty_display_type == 'IMAGE':
                    list.append(ob)
                else:
                    bref_collection.objects.unlink(ob)
                    bpy.context.scene.collection.objects.link(ob)
            if len(list)>0:
                min_x_object = min(list, key=lambda obj: obj.location.x)
                x_min=min_x_object.location.x-xl
            else:
                x_min= 0
                y_max= 0            

        else:
            bref_collection = bpy.data.collections.new(colname)
            data.children.link(bref_collection)
            x_min= 0
            y_max= 0
            
        layer_collection = bpy.context.view_layer.layer_collection.children[colname]
        bpy.context.view_layer.active_layer_collection = layer_collection              
          

        if x not in bpy.data.images:
            img = bpy.data.images.load(filepath)
        else:
            img= bpy.data.images[x]
            print(x,"is recycled")
        index=-1.1
        name="BlendRef_"+x
        if name not in bpy.data.objects:  
            print("empty dos not exist")          
            bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(index+x_min+xl, yl, 0), scale=(1, 1, 1))            
            EMPT=bpy.context.view_layer.objects.active
            EMPT.empty_display_type = 'IMAGE'  
            EMPT.empty_display_size = 1         
            EMPT.data=img           
            EMPT.name="BlendRef_"+x
            EMPT.data.source = 'SEQUENCE'



        else:
            print("empty exists")
        bpy.context.space_data.shading.background_type = 'VIEWPORT'
        my_col= bpy.context.scene.bref_prop.my_col
        bpy.context.space_data.shading.background_color = my_col
        bpy.context.space_data.overlay.show_ortho_grid = False
        bpy.context.space_data.overlay.show_floor = False
        bpy.context.space_data.overlay.show_axis_x = False
        bpy.context.space_data.overlay.show_axis_y = False
        try:
            bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        except:
            pass
        bpy.ops.view3d.view_selected(use_all_regions=False)
        collection = bpy.data.collections["BlendRef-Movies"]
        objects = collection.objects
        for ob in objects:
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob 
        #bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        bpy.ops.view3d.view_selected(use_all_regions=False)
        
        bpy.ops.object.select_all(action='DESELECT')
        bpy.context.view_layer.objects.active = sel
        for o in sels:
            o.select_set(True)
        
        bpy.context.space_data.show_gizmo_empty_image = False
        #lock VIEW3D    
        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = True

        bpy.context.view_layer.active_layer_collection=origin_collection

        bpy.context.space_data.overlay.show_overlays = True
        return{'FINISHED'} 
        
        
        
        
        
        
        
        
        
        
        
        
class ImportVideoOperator(bpy.types.Operator, ImportHelper):
    """Import a video file"""
    bl_idname = "wm.import_video"
    bl_label = "导入视频"
    bl_options = {'REGISTER', 'UNDO'}
    filename_ext = ".mov,.avi,.mp4,.gif,.webm"
    filter_glob: bpy.props.StringProperty(default="*.mov;*.avi;*.mp4;*.gif;*.webm",options={'HIDDEN'})
    def execute(self, context):
        MODE_switch()
        
        
        #bpy.ops.object.mode_set(mode='OBJECT')

        filepath = self.filepath
        print(filepath)
        x = os.path.basename(filepath)
        print(x)

        sel=bpy.context.view_layer.objects.active
        sels = bpy.context.selected_objects
        #folder = os.path.dirname(imgpath)
        data = bpy.context.scene.collection
        colname= "BlendRef-Movies"
        origin_collection= bpy.context.view_layer.active_layer_collection
        col=bpy.context.scene.bref_prop.num
        xl=bpy.context.scene.bref_prop.xloc
        yl=bpy.context.scene.bref_prop.yloc+2        
        if colname in bpy.data.collections:
            list=[]
            d=()
            bref_collection = bpy.data.collections.get(colname)
            objects = bref_collection.objects
            for o in objects:
                list.append(o)
            max_x_object = max(list, key=lambda obj: obj.location.x)
            print(d)
            x_max=max_x_object.location.x-xl
        else:
            bref_collection = bpy.data.collections.new(colname)
            data.children.link(bref_collection)
            x_max=-1.8
            
        layer_collection = bpy.context.view_layer.layer_collection.children[colname]
        bpy.context.view_layer.active_layer_collection = layer_collection              
          

        if x not in bpy.data.images:
            img = bpy.data.images.load(filepath)
        else:
            img= bpy.data.images[x]
            print(x,"is recycled")
        index=2.6
        name="BlendRef_"+x
        if name not in bpy.data.objects:  
            print("empty dos not exist")          
            bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(index+x_max+xl, yl, 0), scale=(1, 1, 1))            
            EMPT=bpy.context.view_layer.objects.active
            EMPT.empty_display_type = 'IMAGE'  
            EMPT.empty_display_size = 2.5         
            EMPT.data=img           
            EMPT.name="BlendRef_"+x
            EMPT.image_user.frame_duration = 100000

        else:
            print("empty exists")
        bpy.context.space_data.shading.background_type = 'VIEWPORT'
        my_col= bpy.context.scene.bref_prop.my_col
        bpy.context.space_data.shading.background_color = my_col
        bpy.context.space_data.overlay.show_ortho_grid = False
        bpy.context.space_data.overlay.show_floor = False
        bpy.context.space_data.overlay.show_axis_x = False
        bpy.context.space_data.overlay.show_axis_y = False
        try:
            bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        except:
            pass
        bpy.ops.view3d.view_selected(use_all_regions=False)
        collection = bpy.data.collections["BlendRef-Movies"]
        objects = collection.objects
        for ob in objects:
            ob.select_set(True)
            bpy.context.view_layer.objects.active = ob 
        #bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        bpy.ops.view3d.view_selected(use_all_regions=False)
        
        bpy.ops.object.select_all(action='DESELECT')
        bpy.context.view_layer.objects.active = sel
        for o in sels:
            o.select_set(True)
        
        
        for ob in objects:
            if ob.name.endswith(".gif"):
                filepath = ob.data.filepath
                name= ob.data.name
                file_format=bpy.data.images[name].file_format
                source= bpy.data.images[name].source 
                bpy.data.images[name].source = 'MOVIE'


        
        bpy.context.space_data.show_gizmo_empty_image = False
        #lock VIEW3D    
        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = True

        bpy.context.view_layer.active_layer_collection=origin_collection

        bpy.context.space_data.overlay.show_overlays = True
        return{'FINISHED'}    



   

          



    
class DELETEIMAGES_button(bpy.types.Operator):
    bl_idname = "wm.deleteimages"
    bl_label = "删除所有图像"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    
    def execute(self, context):  
        my_col= bpy.context.scene.bref_prop.my_col
        colname= "BlendRef-Images"
        if colname in bpy.data.collections:                 
            bref_collection = bpy.data.collections[colname]
            objects = bref_collection.objects
            for ob in objects:
                if ob.type == 'EMPTY' and ob.empty_display_type == 'IMAGE':
                    bpy.data.objects.remove(ob)
                else:
                    bref_collection.objects.unlink(ob)
                    bpy.context.scene.collection.objects.link(ob) 
            bpy.data.collections.remove(bref_collection)
            
        colname2= "BlendRef-Movies"
        if colname2 not in bpy.data.collections:         
            bpy.context.space_data.shading.background_type = 'THEME'
            bpy.context.space_data.shading.background_color = my_col
            bpy.context.space_data.overlay.show_ortho_grid = True
            bpy.context.space_data.overlay.show_floor = True
            bpy.context.space_data.overlay.show_axis_x = True
            bpy.context.space_data.overlay.show_axis_y = True
            bpy.context.space_data.shading.light = 'STUDIO'

            bpy.ops.view3d.view_all(center=False)
            space_data = bpy.context.space_data
            view_location = space_data.region_3d.view_location
            view_rotation = space_data.region_3d.view_rotation
            view_location=1, -1, 1
            view_rotation= 0.9, 0.457, 0, 0 
        
            bpy.context.space_data.show_gizmo_empty_image = True
        #unlock VIEW3D   
            context = bpy.context
            areatype = context.area.type
            print(areatype)
            if areatype=="VIEW_3D":
                bpy.context.area.spaces[0].region_3d.lock_rotation = False
        return {'FINISHED'}  
    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)
        
    
class DELETEMOVIES_button(bpy.types.Operator):
    bl_idname = "wm.deletemovies"
    bl_label = "Delete All Videos"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 
        my_col= bpy.context.scene.bref_prop.my_col 
        colname= "BlendRef-Movies"
        if colname in bpy.data.collections:                 
            bref_collection = bpy.data.collections[colname]
            objects = bref_collection.objects
            for ob in objects:
                if ob.type == 'EMPTY' and ob.empty_display_type == 'IMAGE':
                    bpy.data.objects.remove(ob)
                else:
                    bref_collection.objects.unlink(ob)
                    bpy.context.scene.collection.objects.link(ob) 
            bpy.data.collections.remove(bref_collection)
        colname2= "BlendRef-Images"
        if colname2 not in bpy.data.collections:         
            bpy.context.space_data.shading.background_type = 'THEME'
            bpy.context.space_data.shading.background_color = my_col
            bpy.context.space_data.overlay.show_ortho_grid = True
            bpy.context.space_data.overlay.show_floor = True
            bpy.context.space_data.overlay.show_axis_x = True
            bpy.context.space_data.overlay.show_axis_y = True
            bpy.context.space_data.shading.light = 'STUDIO'

            bpy.ops.view3d.view_all(center=False)
            space_data = bpy.context.space_data
            view_location = space_data.region_3d.view_location
            view_rotation = space_data.region_3d.view_rotation
            view_location=1, -1, 1
            view_rotation= 0.9, 0.457, 0, 0
            
            bpy.context.space_data.show_gizmo_empty_image = True       
            #unlock VIEW3D   
            context = bpy.context
            areatype = context.area.type
            print(areatype)
            if areatype=="VIEW_3D":
                bpy.context.area.spaces[0].region_3d.lock_rotation = False
                
                 
        return {'FINISHED'}  
    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)        



class COLONNES_button(bpy.types.Operator):
    bl_idname = "wm.colonnes"
    bl_label = "全部"
    bl_description="Arrange ALL items in columns"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        col=bpy.context.scene.bref_prop.num
        xl=bpy.context.scene.bref_prop.xloc
        yl=bpy.context.scene.bref_prop.yloc
        collection = bpy.data.collections["BlendRef-Images"]
        objects = collection.objects
        index=-1
        y=0
        list=[ob for ob in objects]
        list.sort(key=lambda ob: ob.location.x) # tri de la list par position en x
        for ob in list:            
            index+=1.1
            if index >col:
                y-=1.1
                index=0.1
            ob.location=index+xl, y+yl, 0
            ob.scale=1,1,1             
                   
        return {'FINISHED'}  
    
class COLONNES2_button(bpy.types.Operator):
    bl_idname = "wm.colonnes2"
    bl_label = "选择的"
    bl_description="Arrange SELECTED items in columns"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):
        col=bpy.context.scene.bref_prop.num
        xl=bpy.context.scene.bref_prop.xloc
        yl=bpy.context.scene.bref_prop.yloc
        collection = bpy.data.collections["BlendRef-Images"]
        objects = collection.objects
        index=-1
        y=0
        if len(bpy.context.selected_objects) >0:        
            list=[ob for ob in objects if ob in bpy.context.selected_objects]       
            min_x = min(list, key=lambda ob: ob.location.x)
            list.sort(key=lambda ob: ob.location.x) # tri de la list par position en x
            for ob in list:
                index+=1.1
                if index >col:
                    y-=1.1
                    index=0.1
                ob.location=index+min_x.location.x, y+min_x.location.y, 0
                ob.scale=1,1,1 
        else:
            pass

        return {'FINISHED'}      
    
    
    
class FLIPX_button(bpy.types.Operator):
    bl_idname = "wm.flipx"
    bl_label = "翻转X"
    bl_description="Horizontal mirror for selected items"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        grp = bpy.context.selected_objects
        try:
            collection = bpy.data.collections["BlendRef-Images"]
            objects = collection.objects
            for o in grp:
                if bpy.data.collections.get("BlendRef-Images") in o.users_collection:
                    sx=o.scale.x
                    o.delta_scale.x = -o.delta_scale.x      
        except:
            pass
        try:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            for o in grp:
                if bpy.data.collections.get("BlendRef-Movies") in o.users_collection:
                    sx=o.scale.x
                    o.delta_scale.x = -o.delta_scale.x      
        except:
            pass          
        return {'FINISHED'}  
class FLIPY_button(bpy.types.Operator):
    bl_idname = "wm.flipy"
    bl_label = "翻转Y"
    bl_description="Vertical mirror for selected items"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        grp = bpy.context.selected_objects
        try:
            collection = bpy.data.collections["BlendRef-Images"]
            objects = collection.objects
            for o in grp:
                if bpy.data.collections.get("BlendRef-Images") in o.users_collection:
                    sy=o.scale.y
                    o.delta_scale.y= -o.delta_scale.y     
        except:
            pass
        try:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            for o in grp:
                if bpy.data.collections.get("BlendRef-Movies") in o.users_collection:
                    sy=o.scale.y
                    o.delta_scale.y= -o.delta_scale.y      
        except:
            pass   
        return {'FINISHED'} 
class UNFLIP_button(bpy.types.Operator):
    bl_idname = "wm.unflip"
    bl_label = "取消翻转"
    bl_description="cancel mirroring for selected items"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        grp = bpy.context.selected_objects
        try:
            collection = bpy.data.collections["BlendRef-Images"]
            objects = collection.objects
            for o in grp:
                if bpy.data.collections.get("BlendRef-Images") in o.users_collection:
                    sy=o.delta_scale.y
                    if sy < 0:
                        o.delta_scale.y= -o.delta_scale.y
                    sx=o.delta_scale.x
                    if sx < 0:
                        o.delta_scale.x= -o.delta_scale.x                
        except:
            pass
        try:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            for o in grp:
                if bpy.data.collections.get("BlendRef-Movies") in o.users_collection:
                    sy=o.delta_scale.y
                    if sy < 0:
                        o.scale.y= -o.delta_scale.y
                    sx=o.delta_scale.x
                    if sx < 0:
                        o.delta_scale.x= -o.delta_scale.x                
        except:
            pass
                
        return {'FINISHED'}  



class FLIPXALL_button(bpy.types.Operator):
    bl_idname = "wm.flipxall"
    bl_label = "翻转全部"
    bl_description="Horizontal mirroring all items"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  

        try:
            collection = bpy.data.collections["BlendRef-Images"]
            objects = collection.objects
            for o in objects:
                if bpy.data.collections.get("BlendRef-Images") in o.users_collection:
                    sx=o.scale.x
                    o.delta_scale.x = -o.delta_scale.x      
        except:
            pass
        try:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            for o in objects:
                if bpy.data.collections.get("BlendRef-Movies") in o.users_collection:
                    sx=o.scale.x
                    o.delta_scale.x = -o.delta_scale.x      
        except:
            pass          
        return {'FINISHED'}  
    
    
     
class FREEZE_button(bpy.types.Operator):
    bl_idname = "wm.freeze"
    bl_label = "冻结帧"
    bl_description="Freeze frame at current time for selected movie"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object
                if ob.name in objects:
                    ob.pass_index = 10
                    f=ob.image_user.frame_offset
                    ob.image_user.frame_duration = 1
                    ob.image_user.frame_start = -1   
                    curr=bpy.context.scene.frame_current
                    ob.image_user.frame_offset = curr+f-1
                else:
                    pass
        else:
            pass    
        return {'FINISHED'}  


    
    
class UNFREEZE_button(bpy.types.Operator):
    bl_idname = "wm.unfreeze"
    bl_label = "取消冻结帧"
    bl_description=" Unfreeze frame for seleted movie"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object
                if ob.name in objects:
                    ob.pass_index = 0
                    f=ob.image_user.frame_offset
                    ob.image_user.frame_duration = 10000
                    ob.image_user.frame_start = 1    
                    ob.image_user.frame_offset = f  
                else:
                    pass
        else:
            pass    
        

        return {'FINISHED'}   

class CLONE_button(bpy.types.Operator):
    bl_idname = "wm.clone"
    bl_label = "克隆影像"
    bl_description="Duplicate selected video"
    def execute(self, context):  
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object
                if ob.name in objects:
                    for o in objects:
                        if o.location.x> ob.location.x:
                            o.location.x= o.location.x+2.6                    
                    ob_copy = ob.copy()
                    collect = bpy.data.collections["BlendRef-Movies"]
                    collect.objects.link(ob_copy)        
                    ob_copy.data = ob_copy.data.copy()
                    ob_copy.location.x=ob.location.x + 2.6
                    ob_copy.name=ob.name+"_copy"                      
                    if ob.name in bpy.data.texts:
                        text_block = bpy.data.texts.get(ob.name) 
                        text_block2 = bpy.data.texts.new(ob_copy.name)
                        for line in text_block.lines:
                            text_block2.write(line.body)

                                      
        return {'FINISHED'}         


#loop A B v1
    
#class A_button(bpy.types.Operator):
#    bl_idname = "wm.a"
#    bl_label = "A"
#    bl_description=""
#    def execute(self, context):  
#        if "BlendRef-Movies" in bpy.data.collections:
#            collection = bpy.data.collections["BlendRef-Movies"]
#            objects = collection.objects
#            if len(bpy.context.selected_objects) >0:
#                ob=bpy.context.object
#                if ob.name in objects:
#                    print("pouet")
#                    A=bpy.context.scene.frame_current
#                    ob.delta_scale[2] = A # juste un endroit ou stocker le chiffre!
#                else:
#                    pass
#        else:
#            pass    
#        return {'FINISHED'}       
#class B_button(bpy.types.Operator):
#    bl_idname = "wm.b"
#    bl_label = "B"
#    bl_description=""
#    def execute(self, context):  
#        if "BlendRef-Movies" in bpy.data.collections:
#            collection = bpy.data.collections["BlendRef-Movies"]
#            objects = collection.objects
#            if len(bpy.context.selected_objects) >0:
#                ob=bpy.context.object
#                if ob.name in objects:
#                    B=bpy.context.scene.frame_current
#                    A=int(ob.delta_scale[2])-1
#                    ob.image_user.frame_offset = A
#                    ob.image_user.frame_duration = B-A
#                    ob.image_user.use_cyclic = True
#                else:
#                    pass
#        else:
#            pass    
#        return {'FINISHED'}    
       
#class UNLOOP_button(bpy.types.Operator):
#    bl_idname = "wm.unloop"
#    bl_label = "Unloop"
#    bl_description=""
#    def execute(self, context):  
#        if "BlendRef-Movies" in bpy.data.collections:
#            collection = bpy.data.collections["BlendRef-Movies"]
#            objects = collection.objects
#            if len(bpy.context.selected_objects) >0:
#                ob=bpy.context.object
#                if ob.name in objects:
#                    ob.delta_scale[2]=1
#                    ob.image_user.use_cyclic = False
#                    ob.image_user.frame_duration = 10000
#                    ob.image_user.frame_start = 1    
#                    ob.image_user.frame_offset = 0  
#                else:
#                    pass
#        else:
#            pass    
#        return {'FINISHED'}     


#************************************************

#loop A B v2
#meme principe que freeze frame
#frame:1   
#start -1   
#offset keyframe = A  
#offset keyframe = B
#SHIFT E make cyclic
class A_button(bpy.types.Operator):
    bl_idname = "wm.a"
    bl_label = "A"
    bl_description="Add entry point at current time for loop"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        if bpy.context.object.pass_index < 1:  
            if "BlendRef-Movies" in bpy.data.collections:
                collection = bpy.data.collections["BlendRef-Movies"]
                objects = collection.objects
                if len(bpy.context.selected_objects) >0:
                    ob=bpy.context.object
                    ob.pass_index = 1
                    if ob.name in objects:
                        A=bpy.context.scene.frame_current
                        v= ob.image_user.frame_offset 
                        if ob.name in bpy.data.texts:
                            text_block = bpy.data.texts.get(ob.name)                       
                        else:
                            text_block = bpy.data.texts.new(ob.name)
                        for line in text_block.lines:
                            line.body = str(A+v)  

                        #marker
                        A=bpy.context.scene.frame_current
                        bpy.context.scene.timeline_markers.new('A', frame=A)

                    else:
                        pass
            else:
                pass    
        else:
            pass 
        return {'FINISHED'}    

class B_button(bpy.types.Operator):
    bl_idname = "wm.b"
    bl_label = "B"
    bl_description="Add out point for loop at current time"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if bpy.context.object.pass_index < 2:            
                if len(bpy.context.selected_objects) >0:
                    ob=bpy.context.object
                    
                    if ob.name in objects:
                        v= ob.image_user.frame_offset  
                        C=bpy.context.scene.frame_current
                        ob.pass_index = 2
                        if ob.name in bpy.data.texts:      
                            text_block = bpy.data.texts[ob.name]
                            for line in text_block.lines:
                                line.body = line.body+" "+str(C+v)
                            text_str = text_block.as_string()
                            list2=[]
                            for num in text_str.split():
                                list2.append(int(num))
                            A=list2[0]
                            B=list2[1]  
                                        
                            ob.image_user.frame_duration = 1
                            ob.image_user.frame_start = -1                         
                            bpy.context.scene.frame_set(A-v)
                            ob.image_user.frame_offset = A
                            ob.image_user.keyframe_insert("frame_offset")
                            bpy.context.scene.frame_set(B-v)
                            ob.image_user.frame_offset = B
                            ob.image_user.keyframe_insert("frame_offset")
                            
                            # remove marker
                            for marker in bpy.context.scene.timeline_markers:
                                if marker.name=='A':
                                    bpy.context.scene.timeline_markers.remove(marker)
                            
                            context = bpy.context
                            areatype = context.area.type
                            bpy.context.area.type = 'DOPESHEET_EDITOR'
                            bpy.context.space_data.mode = 'ACTION'
                            bpy.ops.action.extrapolation_type(type='MAKE_CYCLIC')
                            context.area.type = areatype
                        else:
                            pass                    
                    else:
                        pass
            else:
                pass 
        else:
            pass   
        return {'FINISHED'}    
    
        

    
    
    
    
    
    
    
class SPEED1_button(bpy.types.Operator):
    bl_idname = "wm.speed1"
    bl_label = "X0.5"
    bl_description="Decrease speed by 2"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object                
                if ob.name in objects:
                    B=bpy.context.scene.frame_current
                    ob.pass_index = 3
                    if ob.name in bpy.data.texts:      
                        text_block = bpy.data.texts[ob.name]
                        text_str = text_block.as_string()
                        list2=[]
                        for num in text_str.split():
                            list2.append(int(num))
                        A=list2[0]
                        B=list2[1]   
                        D=B-A
                        bpy.context.scene.frame_set(0)
                        bpy.ops.screen.keyframe_jump(next=True)   
                                                          
                        ob.image_user.frame_duration = 1
                        ob.image_user.frame_start = -1   
                        ob.animation_data_clear()                       
                        ob.image_user.frame_offset = A
                        ob.image_user.keyframe_insert("frame_offset")
                        
                        bpy.context.scene.frame_set(bpy.context.scene.frame_current+D+D)                 
                        ob.image_user.frame_offset = B
                        ob.image_user.keyframe_insert("frame_offset")
                        
                        context = bpy.context
                        areatype = context.area.type
                        bpy.context.area.type = 'DOPESHEET_EDITOR'
                        bpy.context.space_data.mode = 'ACTION'
                        bpy.ops.action.extrapolation_type(type='MAKE_CYCLIC')
                        context.area.type = areatype
                        if bpy.context.scene.use_preview_range == True:
                            bpy.context.scene.frame_set(0)
                            bpy.ops.screen.keyframe_jump(next=True)
                            A=bpy.context.scene.frame_current
                            bpy.ops.screen.keyframe_jump(next=True)  
                            B=bpy.context.scene.frame_current       
                            bpy.context.scene.use_preview_range = True
                            bpy.context.scene.frame_preview_start = A
                            bpy.context.scene.frame_preview_end = B
                    else:
                        pass                    
                else:
                    pass
        else:
            pass    
        return {'FINISHED'}  
    
class SPEED2_button(bpy.types.Operator):
    bl_idname = "wm.speed2"
    bl_label = "X0.33"
    bl_description="Decrease speed by 3"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object                
                if ob.name in objects:
                    B=bpy.context.scene.frame_current
                    ob.pass_index = 4
                    if ob.name in bpy.data.texts:      
                        text_block = bpy.data.texts[ob.name]
                        text_str = text_block.as_string()
                        list2=[]
                        for num in text_str.split():
                            list2.append(int(num))
                        A=list2[0]
                        B=list2[1]   
                        D=B-A
                        bpy.context.scene.frame_set(0)
                        bpy.ops.screen.keyframe_jump(next=True)   
                                                          
                        ob.image_user.frame_duration = 1
                        ob.image_user.frame_start = -1   
                        ob.animation_data_clear()                       
                        ob.image_user.frame_offset = A
                        ob.image_user.keyframe_insert("frame_offset")
                        
                        bpy.context.scene.frame_set(bpy.context.scene.frame_current+D+D+D)                 
                        ob.image_user.frame_offset = B
                        ob.image_user.keyframe_insert("frame_offset")
                        
                        context = bpy.context
                        areatype = context.area.type
                        bpy.context.area.type = 'DOPESHEET_EDITOR'
                        bpy.context.space_data.mode = 'ACTION'
                        bpy.ops.action.extrapolation_type(type='MAKE_CYCLIC')
                        context.area.type = areatype
                        if bpy.context.scene.use_preview_range == True:
                            bpy.context.scene.frame_set(0)
                            bpy.ops.screen.keyframe_jump(next=True)
                            A=bpy.context.scene.frame_current
                            bpy.ops.screen.keyframe_jump(next=True)  
                            B=bpy.context.scene.frame_current       
                            bpy.context.scene.use_preview_range = True
                            bpy.context.scene.frame_preview_start = A
                            bpy.context.scene.frame_preview_end = B
                    else:
                        pass                    
                else:
                    pass
        else:
            pass    
        return {'FINISHED'}  
    
      
class SPEED0_button(bpy.types.Operator):
    bl_idname = "wm.speed0"
    bl_label = "X1"
    bl_description="Set to normal speed"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object                
                if ob.name in objects:
                    B=bpy.context.scene.frame_current
                    ob.pass_index = 2
                    if ob.name in bpy.data.texts:      
                        text_block = bpy.data.texts[ob.name]
                        text_str = text_block.as_string()
                        list2=[]
                        for num in text_str.split():
                            list2.append(int(num))
                        A=list2[0]
                        B=list2[1]   
                        D=B-A
                        bpy.context.scene.frame_set(0)
                        bpy.ops.screen.keyframe_jump(next=True)   
                                                          
                        ob.image_user.frame_duration = 1
                        ob.image_user.frame_start = -1   
                        ob.animation_data_clear()                       
                        ob.image_user.frame_offset = A
                        ob.image_user.keyframe_insert("frame_offset")
                        
                        bpy.context.scene.frame_set(bpy.context.scene.frame_current+D)                 
                        ob.image_user.frame_offset = B
                        ob.image_user.keyframe_insert("frame_offset")
                        
                        context = bpy.context
                        areatype = context.area.type
                        bpy.context.area.type = 'DOPESHEET_EDITOR'
                        bpy.context.space_data.mode = 'ACTION'
                        bpy.ops.action.extrapolation_type(type='MAKE_CYCLIC')
                        context.area.type = areatype
                        if bpy.context.scene.use_preview_range == True:
                            bpy.context.scene.frame_set(0)
                            bpy.ops.screen.keyframe_jump(next=True)
                            A=bpy.context.scene.frame_current
                            bpy.ops.screen.keyframe_jump(next=True)  
                            B=bpy.context.scene.frame_current       
                            bpy.context.scene.use_preview_range = True
                            bpy.context.scene.frame_preview_start = A
                            bpy.context.scene.frame_preview_end = B
                    else:
                        pass                    
                else:
                    pass
        else:
            pass    
        return {'FINISHED'}     

class UNLOOP_button(bpy.types.Operator):
    bl_idname = "wm.unloop"
    bl_label = "不循环"
    bl_description="Disable loop"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object
                ob.pass_index = 0
                if ob.name in objects:
                    ob.delta_scale[2]=1
                    ob.image_user.use_cyclic = False
                    ob.image_user.frame_duration = 10000
                    ob.image_user.frame_start = 1  
                    f= bpy.context.scene.bref_prop.frame1  
                    ob.image_user.frame_offset = f  
                    ob.animation_data_clear() 
                    if bpy.context.scene.use_preview_range == True:
                        bpy.context.scene.use_preview_range = False

                else:
                    pass
        else:
            pass    
        # remove marker
        for marker in bpy.context.scene.timeline_markers:
            if marker.name=='A':
                bpy.context.scene.timeline_markers.remove(marker)
        

        return {'FINISHED'}    
    
class RANGE_button(bpy.types.Operator):
    bl_idname = "wm.range"
    bl_label = "预览排列"
    bl_description="Enable preview range on loop"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        if bpy.context.scene.use_preview_range == False:
            if "BlendRef-Movies" in bpy.data.collections:
                collection = bpy.data.collections["BlendRef-Movies"]
                objects = collection.objects
                if len(bpy.context.selected_objects) >0:
                    ob=bpy.context.object                
                    if ob.name in objects:
                        bpy.context.scene.frame_set(0)
                        bpy.ops.screen.keyframe_jump(next=True)
                        A=bpy.context.scene.frame_current
                        bpy.ops.screen.keyframe_jump(next=True)  
                        B=bpy.context.scene.frame_current       
                        bpy.context.scene.use_preview_range = True
                        bpy.context.scene.frame_preview_start = A
                        bpy.context.scene.frame_preview_end = B
        else:
            bpy.context.scene.use_preview_range = False
        return {'FINISHED'}    
    
class OFFSET_button(bpy.types.Operator):
    bl_idname = "wm.offset"
    bl_label = "应用"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 

        if "BlendRef-Movies" in bpy.data.collections:
            collection = bpy.data.collections["BlendRef-Movies"]
            objects = collection.objects
            if len(bpy.context.selected_objects) >0:
                ob=bpy.context.object                
                if ob.name in objects:
                    f= bpy.context.scene.bref_prop.frame1
                    ob.image_user.frame_offset = f
                    bpy.context.region.tag_redraw()
                    

         
        return {'FINISHED'}        

 
class REFRESH_button(bpy.types.Operator):
    bl_idname = "wm.refresh"
    bl_label = "中心视图"
    bl_description="Refresh Blendref display and center the view to reference items"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        for ob in bpy.data.objects:
            if ob.name.startswith("BlendRef_"):        
                bpy.context.space_data.shading.background_type = 'VIEWPORT'
                my_col= bpy.context.scene.bref_prop.my_col
                bpy.context.space_data.shading.background_color = my_col
                bpy.context.space_data.overlay.show_ortho_grid = False
                bpy.context.space_data.overlay.show_floor = False
                bpy.context.space_data.overlay.show_axis_x = False
                bpy.context.space_data.overlay.show_axis_y = False
                try:
                    bpy.ops.view3d.view_axis(type="TOP", align_active=True)
                except:
                    pass      
                #lock VIEW3D    
                context = bpy.context
                areatype = context.area.type
                print(areatype)
                if areatype=="VIEW_3D":
                    bpy.context.area.spaces[0].region_3d.lock_rotation = True
              



        if bpy.context.object == None:
            if bpy.context.view_layer.objects != None:
                for ob in bpy.context.view_layer.objects:
                    bpy.context.view_layer.objects.active = ob
                    startmode = bpy.context.object.mode
            else:
                startmode = 'OBJECT'
        else:
            startmode = bpy.context.object.mode
            
            
            
            
            
        bpy.ops.object.mode_set(mode='OBJECT') 
        activ_ob=bpy.context.view_layer.objects.active
        grp = bpy.context.selected_objects
        bpy.ops.object.select_all(action='DESELECT')     
        for ob in bpy.data.objects:
            if ob.name.startswith("BlendRef_"):
                ob.select_set(True)
        bpy.ops.view3d.view_selected(use_all_regions=False)
        bpy.ops.object.select_all(action='DESELECT') 
        for ob in grp:
            ob.select_set(True)
            
        for o in bpy.context.view_layer.objects:    
            if o == activ_ob:
                bpy.context.view_layer.objects.active = activ_ob
        
        bpy.ops.object.mode_set(mode=startmode) 
        bpy.context.space_data.show_gizmo_empty_image = False  
        bpy.context.space_data.overlay.show_overlays = True  
        return {'FINISHED'}      

class UPDATE_button(bpy.types.Operator):
    bl_idname = "wm.update"
    bl_label = "应用"
    bl_description="Apply background color"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        my_col= bpy.context.scene.bref_prop.my_col
        bpy.context.space_data.shading.background_color = my_col           
        return {'FINISHED'}  




class RESTORE_button(bpy.types.Operator):
    bl_idname = "wm.restore"
    bl_label = "恢复视图"
    bl_description="Restore default blender viewport settings"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        bpy.context.space_data.shading.background_type = 'THEME'
        bpy.context.space_data.overlay.show_ortho_grid = True
        bpy.context.space_data.overlay.show_floor = True
        bpy.context.space_data.overlay.show_axis_x = True
        bpy.context.space_data.overlay.show_axis_y = True
        bpy.ops.view3d.view_all(center=False)
        space_data = bpy.context.space_data
        view_location = space_data.region_3d.view_location
        view_rotation = space_data.region_3d.view_rotation
        view_location=1, -1, 1
        view_rotation= 0.9, 0.457, 0, 0    
        #unlock VIEW3D   
        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = False 
        bpy.context.space_data.show_gizmo_empty_image = True 
        return {'FINISHED'}  


class DRAG_button(bpy.types.Operator):
    bl_idname = "wm.drag"
    bl_label = "应用当前视图"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context): 

        if in_BREF_coll()==True:
            # duplicate from blendref
            ob=bpy.context.view_layer.objects.active
            ob2 = ob.copy()
            collect = bpy.context.scene.collection
            collect.objects.link(ob2)  
            ob2.name="V_"+ob.name                    
            # create parent (for local transformations)
            
            ob2_copy = ob2.copy()
            ob2_copy.name="TEMP_PARENT"
            collect.objects.link(ob2_copy)        
            ob2.parent=ob2_copy
            ob2.matrix_parent_inverse = ob2_copy.matrix_world.inverted() 
            #apply view transformation to parent                   
            ob2_copy.rotation_mode = 'QUATERNION'
            r3d = bpy.context.area.spaces.active.region_3d                    
            ob2_copy.location=r3d.view_matrix.inverted().translation
            #apply view orientation to parent
            ob2_copy.rotation_quaternion= r3d.view_rotation  
            # empty settings              
            ob2.use_empty_image_alpha = True
            ob2.color[3] = 0.5    
            #local Z child translation            
            vec = mathutils.Vector((0.0, 0.0, -5))
            inv = ob2.matrix_world.copy()
            inv.invert()
            vec_rot = vec @ inv
            ob2.location = ob2.location + vec_rot 
            # remove parent                   
            bpy.context.view_layer.objects.active = ob2
            bpy.ops.object.parent_clear(type='CLEAR_KEEP_TRANSFORM')
            bpy.data.objects.remove(ob2_copy, do_unlink=True)

            bpy.ops.object.select_all(action='DESELECT')
            ob2.select_set(True)
            bpy.context.view_layer.objects.active = ob2
            
#                    collect.objects.unlink(ob2)
#                    bpy.context.scene.collection.objects.link(ob2)


            
        return {'FINISHED'}  

class ENABLE_button(bpy.types.Operator):
    bl_idname = "wm.enable"
    bl_label = "开启"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        sel=bpy.context.view_layer.objects.active
        sels = bpy.context.selected_objects
        xl=bpy.context.scene.bref_prop.xloc
        yl=bpy.context.scene.bref_prop.yloc
        MODE_switch()

       

#        try:   
#            bpy.ops.object.select_all(action='DESELECT')
#        except:
#            pass
#        if bpy.context.object == None:
#            if bpy.context.view_layer.objects != None:
#                for ob in bpy.context.view_layer.objects:
#                    bpy.context.view_layer.objects.active = ob
#                    startmode = bpy.context.object.mode
#            else:
#                startmode = 'OBJECT'
#        else:
#            startmode = bpy.context.object.mode   
                          
        bpy.context.space_data.shading.background_type = 'VIEWPORT'
        my_col= bpy.context.scene.bref_prop.my_col
        bpy.context.space_data.shading.background_color = my_col
        bpy.context.space_data.overlay.show_ortho_grid = False
        bpy.context.space_data.overlay.show_floor = False
        bpy.context.space_data.overlay.show_axis_x = False
        bpy.context.space_data.overlay.show_axis_y = False
        bpy.context.space_data.show_gizmo_empty_image = False 
        bpy.context.space_data.overlay.show_overlays = True 
        bpy.context.space_data.shading.light = 'FLAT'

    
#create empty on blendref location
        bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(xl, yl, 0), scale=(1, 1, 1))            
        EMPT=bpy.context.view_layer.objects.active   
                
        bpy.ops.view3d.view_selected(use_all_regions=False)
#delete empty
        bpy.data.objects.remove(EMPT, do_unlink=True)
        try:
            bpy.ops.view3d.view_axis(type="TOP", align_active=True)
        except:
            pass   
        bpy.ops.view3d.view_axis(type="TOP", align_active=True) 
        #lock VIEW3D    
        context = bpy.context
        areatype = context.area.type
        #print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = True

   

     
        return {'FINISHED'}  
class DISABLE_button(bpy.types.Operator):
    bl_idname = "wm.disable"
    bl_label = "关闭"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        bpy.context.space_data.shading.background_type = 'THEME'
        bpy.context.space_data.overlay.show_ortho_grid = True
        bpy.context.space_data.overlay.show_floor = True
        bpy.context.space_data.overlay.show_axis_x = True
        bpy.context.space_data.overlay.show_axis_y = True
        bpy.context.space_data.shading.light = 'STUDIO'

        bpy.ops.view3d.view_all(center=False)
        space_data = bpy.context.space_data
        view_location = space_data.region_3d.view_location
        view_rotation = space_data.region_3d.view_rotation
        view_location=1, -1, 1
        view_rotation= 0.9, 0.457, 0, 0    
        #unlock VIEW3D   
        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = False 
        bpy.context.space_data.show_gizmo_empty_image = True 
        return {'FINISHED'}  
    
#class SAVETO_button(bpy.types.Operator):
#    bl_idname = "wm.saveto"
#    bl_label = "Save a copy to ..."
#    bl_description=""
#    bl_options = {'REGISTER', 'UNDO'}
#    def execute(self, context):  
#        B_path="/home/lo/Pictures/test"
#        grp = bpy.context.selected_objects

#        collection = bpy.data.collections["BlendRef-Images"]
#        objects = collection.objects
#        for o in grp:
#            if bpy.data.collections.get("BlendRef-Images") in o.users_collection:
#                img_name = o.data.name
#                img=bpy.data.images[img_name]
#                img.save_render(os.path.join(B_path, img_name))
#        return {'FINISHED'}      




class SAVETO_button(Operator, ImportHelper): 
    bl_idname = "wm.saveto" 
    bl_label = "保存副本到..." 
    #filter_glob: StringProperty(default='*.jpg',options={'HIDDEN'})
    filename_ext = bpy.props.StringProperty(default=".jpg")
    def execute(self, context): 
        sc =bpy.context.scene
        filepath = self.filepath
        grp = bpy.context.selected_objects

        collection = bpy.data.collections["BlendRef-Images"]
        objects = collection.objects
        for o in grp:
            if bpy.data.collections.get("BlendRef-Images") in o.users_collection:
                img_name = o.data.name
                img=bpy.data.images[img_name]
                
                img.save_render(os.path.join(filepath, img_name))
     
        
        
        
        return {'FINISHED'} 



    
    
class UNLCK_button(bpy.types.Operator):
    bl_idname = "wm.unlck"
    bl_label = "解锁"
    bl_description="Viewport Render. Save it from Image Editor window, Render Result Image"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  

        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = False 
        return {'FINISHED'}  
class LCK_button(bpy.types.Operator):
    bl_idname = "wm.lck"
    bl_label = "加锁"
    bl_description="Viewport Render. Save it from Image Editor window, Render Result Image"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  

        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = True
        return {'FINISHED'}      

# screenshot basic
#class SCREENSHOT_button(bpy.types.Operator):
#    bl_idname = "wm.screenshot"
#    bl_label = "Screenshot"
#    bl_description="Viewport Render. Save it from Image Editor window, Render Result Image"
#    bl_options = {'REGISTER', 'UNDO'}
#    def execute(self, context):  

#        context = bpy.context
#        areatype = context.area.type
#        print(areatype)
#        if areatype=="VIEW_3D":
#            bpy.context.area.spaces[0].region_3d.lock_rotation = False 
#            
#            bpy.ops.render.opengl()
#            bpy.context.area.spaces[0].region_3d.lock_rotation = True 
#        return {'FINISHED'}  





#screenshot advanced    
class SCREENSHOT_button(bpy.types.Operator):
    bl_idname = "wm.screenshot"
    bl_label = "屏幕截图"
    bl_description="Viewport Render. Save it from Image Editor window, Render Result Image"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  

        context = bpy.context
        areatype = context.area.type
        print(areatype)
        objs = bpy.data.objects
        for o in objs:
            if o.name=="blendrefcam":
                objs.remove(objs["blendrefcam"], do_unlink=True)
        if areatype=="VIEW_3D":
            bpy.context.area.spaces[0].region_3d.lock_rotation = False 
            xl=bpy.context.scene.bref_prop.xloc
            yl=bpy.context.scene.bref_prop.yloc            
            #camera2 creation
            cam = bpy.ops.object.camera_add(location=(xl,yl,10),rotation=(0,0,0))
            cam = bpy.context.active_object
            cam.name = "blendrefcam"
            cam.data.name = "blendrefcam"
            localZ= 0.2
            bpy.ops.transform.translate(value=(-0, 0, localZ), orient_type='LOCAL')
            bpy.data.cameras['blendrefcam'].type = 'ORTHO'
            #add driver to zoom
            for o in bpy.data.objects:
                if o.name=="blendrefcam":              
                    path1=str("bref_prop.zoom")          
                    drv = o.data.driver_add("ortho_scale")
                    drv.driver.expression = 'var'
                    var = drv.driver.variables.new()
                    var.name = "var"
                    var.type='SINGLE_PROP'
                    var.targets[0].id_type = 'SCENE' #
                    var.targets[0].id = bpy.context.scene
                    var.targets[0].data_path=path1
            # read previous camera location  from txt  
            camtxt="blendrefcam"        
            if camtxt in bpy.data.texts:
                text_block = bpy.data.texts[camtxt]
                text_str = text_block.as_string()
                list2=[]                
                for num in text_str.split():
                    list2.append(float(num))
                camX=list2[4]
                camY=list2[5] 
                camZ=list2[6]   
                cam.location[0] = camX
                cam.location[1] = camY
                cam.location[2] = camZ             
            #set as active camera
            bpy.context.scene.camera =cam
            #
            #write memo to restore render resolution later
            rezY=bpy.context.scene.render.resolution_y
            rezX=bpy.context.scene.render.resolution_x
            cam="blendrefcam"
            if cam in bpy.data.texts:
                text_block = bpy.data.texts.get(cam) 
            # read previous screenshot size from txt 
                text_block = bpy.data.texts[cam]
                text_str = text_block.as_string()
                list2=[]                
                for num in text_str.split():
                    list2.append(float(num))
                temprezX=int(list2[2])
                temprezY=int(list2[3])  
                bpy.context.scene.render.resolution_x=temprezX
                bpy.context.scene.render.resolution_y=temprezY 
            #######################                    
            else:
                text_block = bpy.data.texts.new(cam)
            for line in text_block.lines:
                line.body = str(rezX)+" "+str(rezY) 
            ########
            bpy.ops.view3d.view_camera()
            bpy.context.object.data.type = 'ORTHO'

            
        return {'FINISHED'}      


class SCREENSHOT2_button(bpy.types.Operator):
    bl_idname = "wm.screenshot2"
    bl_label = "渲染"
    bl_description="Viewport Render. Save it from Image Editor window, Render Result Image"
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):  
        temprezY=bpy.context.scene.render.resolution_y
        temprezX=bpy.context.scene.render.resolution_x
        context = bpy.context
        areatype = context.area.type
        print(areatype)
        if areatype=="VIEW_3D":
            # viewport render
            bpy.context.space_data.shading.show_object_outline = False
            bpy.ops.render.opengl()
            bpy.context.space_data.shading.show_object_outline = True
            bpy.context.area.spaces[0].region_3d.lock_rotation = True 

            # read memo camera resolutions
            camtxt="blendrefcam"
            if camtxt in bpy.data.texts:      
                text_block = bpy.data.texts[camtxt]
                text_str = text_block.as_string()
                for line in text_block.lines:
                    line.body = line.body+" "+str(temprezX)+" "+str(temprezY) 
                list2=[]
                for num in text_str.split():
                    list2.append(float(num))
                rezX=int(list2[0])
                rezY=int(list2[1])  
                bpy.context.scene.render.resolution_x=rezX
                bpy.context.scene.render.resolution_y=rezY 
                 
            #write camera location to txt for next screenshot
            cam="blendrefcam"
            camera = bpy.data.objects.get(cam)
            cX=camera.location[0]
            cY=camera.location[1]
            cZ=camera.location[2]
            if cam in bpy.data.texts:
                text_block = bpy.data.texts.get(cam) 
                for line in text_block.lines:
                    line.body = line.body+" "+str(cX)+" "+str(cY)+" "+str(cZ)             
            #kill blenderefcam
            objs = bpy.data.objects
            for o in objs:
                if o.name=="blendrefcam":
                    objs.remove(objs["blendrefcam"], do_unlink=True)
        return {'FINISHED'}  



class TXT_button(bpy.types.Operator):
    bl_idname = "wm.txt"
    bl_label = "添加文本"
    bl_description=""
    bl_options = {'REGISTER', 'UNDO'}
    def execute(self, context):

        data = bpy.context.scene.collection
        collname= "BlendRef-Text"
        origin_collection= bpy.context.view_layer.active_layer_collection
        
        for c in bpy.data.collections:            
            print(c.name)
            
        if collname in bpy.data.collections:
          
            bref_collection = bpy.data.collections.get(collname)
            print("NO")
        else:
            bref_collection = bpy.data.collections.new(collname)
            data.children.link(bref_collection)
            print("YES")
            
        layer_collection = bpy.context.view_layer.layer_collection.children[collname]
        bpy.context.view_layer.active_layer_collection = layer_collection          
            
            
        cursor_loc= bpy.context.scene.cursor.location
        #create empty on 3d cursor location
        bpy.ops.object.empty_add(type='PLAIN_AXES', align='WORLD', location=(cursor_loc), scale=(1, 1, 1))            
        EMPT=bpy.context.view_layer.objects.active   

        r3d = bpy.context.area.spaces.active.region_3d
        view_matrix = r3d.view_matrix.inverted().translation 
        bpy.context.scene.cursor.location=view_matrix
        bpy.context.scene.cursor.location[2]=0
        bpy.ops.object.text_add()
        TXT=bpy.context.view_layer.objects.active
        TXT.name="BlendRef_TXT"
        
        
        bpy.context.scene.cursor.location=EMPT.location
        #delete empty
        bpy.data.objects.remove(EMPT, do_unlink=True)

        return {'FINISHED'}   
    
    
    
                                            
# ------------------------------------------------------------------------
#    Panel in Object Mode
# ------------------------------------------------------------------------

class blendrefMAINPanel:

    bl_space_type = "VIEW_3D"   
    bl_region_type = "UI"
    bl_category = "BlendRef"
    
class BLENDREF_PT_PANEL(blendrefMAINPanel,Panel):
    bl_label = "BlendRef"
    bl_idname = "BLENDREF_PT_PANEL"  
    def draw(self, context):
        
        layout = self.layout        
        scene = context.scene
        bref_prop = scene.bref_prop  
           

        if bpy.context.area.spaces[0].region_3d.lock_rotation != True:
            row = layout.row()
            col = row.column()
            row.operator("wm.enable",depress=False,icon='RESTRICT_VIEW_ON')  
            #row.operator("wm.disable",text="Disabled",depress=True)  
            if in_BREF_coll()==True:         
                row = layout.row()
                col = row.column()
                row.operator("wm.drag")
                row.scale_y = 1.6 
        else:
            row = layout.row()
            col = row.column()
            row.operator("wm.disable",text="关闭",depress=True,icon='RESTRICT_VIEW_OFF')  
            row = layout.row()
            col = row.column()
            row.operator("wm.txt",icon='OUTLINER_DATA_FONT')  
            row.operator("wm.screenshot",icon='RESTRICT_RENDER_OFF')  
            

   
#            row = layout.row()
#            col = row.column()
#            row.operator("wm.lck",icon='LOCKED')
#            row.operator("wm.unlck",icon='UNLOCKED')
            #row.operator("wm.disable",depress=False)           

        for o in bpy.data.objects:
            if o.name=="blendrefcam": 
                row = layout.row()
                col = row.column()                  
             
                row.label(text="渲染设置:")    
                row = layout.row()
                col = row.column()      
                row.prop(bref_prop,"zoom")   
#                row = layout.row() 
#                col = row.column()  
#                row.prop(bref_prop,"rezx")                                                     
#                row.prop(bref_prop,"rezy")                        
#                row = layout.row()
#                col = row.column()      
#                row.prop(bref_prop,"percent")  
                
                row = layout.row()
                col = row.column()
                row.operator("wm.screenshot2",icon='RESTRICT_RENDER_OFF') 
                row.scale_y=2                 
                             
class BRIMAGE_PT_PANEL(blendrefMAINPanel,Panel):
    bl_parent_id = "BLENDREF_PT_PANEL"
    bl_label = "图像"
    bl_idname = "BRIMAGE_PT_PANEL"
    bl_options = {"DEFAULT_CLOSED"}



    def draw(self, context):
        layout = self.layout
        scene = context.scene
        bref_prop = scene.bref_prop
        if bpy.context.area.spaces[0].region_3d.lock_rotation == True:
    #   IMAGES     
            box = layout.box()
            split = box.split()
            col = split.column(align=True)  
            row = col.row(align=True) 
            row.prop(bref_prop, "path", text="")  
            row = col.row(align=True)  
            row.operator("wm.importfolder",icon='FILE_FOLDER') 
            row.scale_y=2  
            row = col.row(align=True)  
            row.operator("wm.import_image",icon='FILE_IMAGE') 
            row.scale_y=2  
            col.separator()  
            if "BlendRef-Images" not in bpy.data.collections:
                row = col.row(align=True)
                row.label(text="导入图片...")            
            else:   
                         
                if bpy.context.object != None:
                    ob=bpy.context.object
                    img= str(ob.name[9:]) 
                try:     
                    if ob.type == 'EMPTY' and ob.empty_display_type == 'IMAGE':
                        x=ob.data.filepath 
                        for word in ['.jpg', '.jpeg', '.png']:
                            if not x.endswith(word) or x.endswith(word.upper()):
                                continue
                            else:         
                                row = col.row(align=True)
                                row.prop(bref_prop,"num")
                                row.scale_x = 0.4
                                row = col.row(align=True)
                                row.operator("wm.colonnes") 
                                row.operator("wm.colonnes2")
                                col.separator() 
                                row = col.row(align=True)
                         
                                row = col.row(align=True)  
                                
                                if bpy.context.object.delta_scale[0] == -1:
                                    row.operator("wm.flipx",depress=True,icon='RIGHTARROW_THIN') 
                                else:
                                    row.operator("wm.flipx",depress=False,icon='RIGHTARROW_THIN') 
                                    
                                if bpy.context.object.delta_scale[1] == -1:
                                    row.operator("wm.flipy",depress=True,icon='DOWNARROW_HLT') 
                                    
                                else:
                                    row.operator("wm.flipy",depress=False,icon='DOWNARROW_HLT') 
                                     
                                row.operator("wm.unflip")                                
                                    
                                filepath = ob.data.filepath
                                name= ob.data.name
                                file_format=bpy.data.images[name].file_format
                                sizeX=bpy.data.images[name].size[0]
                                sizeY=bpy.data.images[name].size[1] 
                                row = col.row(align=True)                            
                                row.label(text= "信息:", icon= 'INFO')
                                row = col.row(align=True) 
                                row.label(text="") 
                                row.scale_x = 3                             
                                row.label(text= "文件格式: "+str(file_format))
                                row = col.row(align=True)   
                                row.label(text="") 
                                row.scale_x = 3  
                                row.label(text= "大小: "+str(sizeX)+"x"+str(sizeY))
                    
    #                    else:
    #                        row = col.row(align=True)
    #                        row.label(text="Select an image...")  
    #                if ob.type != 'EMPTY':
    #                    row = col.row(align=True)
    #                    row.label(text="Select an image...")                                         
     
                except:
                    pass
    #                row = col.row(align=True)
    #                row.label(text="Select an image...")     




                row = col.row(align=True) 
                row.operator("wm.flipxall",icon='RIGHTARROW_THIN') 
                row = col.row(align=True) 
                row.operator("wm.saveto",icon='FILE_TICK') 
                col.separator()  
                row = col.row(align=True) 
                row.operator("wm.deleteimages",icon='ERROR')    
        
#   MOVIES  
class BRMOVIE_PT_PANEL(blendrefMAINPanel,Panel):
    bl_parent_id = "BLENDREF_PT_PANEL"
    bl_label = "影片"
    bl_options = {"DEFAULT_CLOSED"}
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        bref_prop = scene.bref_prop 
        if bpy.context.area.spaces[0].region_3d.lock_rotation == True:
            box = layout.box()
            split = box.split()
            col = split.column(align=True)  


            row = col.row(align=True)  
            row.operator("wm.import_video",icon='FILE_MOVIE') 
            row.scale_y=2   
            
            row = col.row(align=True)  
            row.operator("wm.import_image_seq",icon='FILE_MOVIE') 
            row.scale_y=2
            
            col.separator()  
            if "BlendRef-Movies" not in bpy.data.collections:
                row = col.row(align=True)
                row.label(text="导入视频...")
            else:   
                         
                if bpy.context.object != None:
                    ob=bpy.context.object
                    
                    movies_collection = bpy.data.collections.get("BlendRef-Movies")
                    mobjects = movies_collection.objects
                    list=[]
                    for o in movies_collection.objects:
                        list.append(o)
                
                    #if ob in movies_collection:
                    if ob.type == 'EMPTY' and ob in list:#and ob.empty_display_type == 'IMAGE':
                        x=ob.data.filepath  
                        for word in ['.mp4', '.avi', '.webm', '.gif', '.jpg','.png']:
                            if not x.endswith(word) or x.endswith(word.upper()):
                                continue
                            else:

                                row = col.row(align=True)  
                                if bpy.context.object.delta_scale[0] == -1:
                                    row.operator("wm.flipx",depress=True,icon='RIGHTARROW_THIN') 
                                else:
                                    row.operator("wm.flipx",depress=False,icon='RIGHTARROW_THIN') 
                                    
                                if bpy.context.object.delta_scale[1] == -1:
                                    row.operator("wm.flipy",depress=True,icon='DOWNARROW_HLT') 
                                else:
                                    row.operator("wm.flipy",depress=False,icon='DOWNARROW_HLT') 
                                row.operator("wm.unflip")
                                if bpy.context.area.spaces[0].region_3d.lock_rotation == True:  
                                    row = col.row(align=True)  
                                    row.operator("wm.clone",icon='DUPLICATE') 
                                row = col.row(align=True)  
                                if bpy.context.object.pass_index == 10:
                                    row.operator("wm.freeze",depress=True,icon='PAUSE') 
                                else:
                                    row.operator("wm.freeze",depress=False,icon='PAUSE')    
                                row.operator("wm.unfreeze") 
                                col.separator() 
                                row = col.row(align=True)
                                row.prop(bref_prop,"frame1")
                                row.scale_x = 0.4
                                row.operator("wm.offset",depress=False)                        
                                row = col.row(align=True)  
                                if bpy.context.object.pass_index > 0 and bpy.context.object.pass_index < 10:
                                    row.operator("wm.a",depress=True,icon='PINNED') 
                                else:
                                    row.operator("wm.a",depress=False,icon='UNPINNED') 
                                    
                                if bpy.context.object.pass_index >1 and bpy.context.object.pass_index < 10:
                                    row.operator("wm.b",depress=True,icon='PINNED') 
                                else:
                                    row.operator("wm.b",depress=False,icon='UNPINNED') 
                                
                                row = col.row(align=True) 
                                if bpy.context.object.pass_index == 2:
                                    row.operator("wm.speed0",depress=True) 
                                else:
                                    row.operator("wm.speed0",depress=False)  
                                    
                                if bpy.context.object.pass_index == 3:
                                    row.operator("wm.speed1",depress=True) 
                                else:
                                    row.operator("wm.speed1",depress=False)  
                                if bpy.context.object.pass_index == 4:
                                    row.operator("wm.speed2",depress=True) 
                                else:
                                    row.operator("wm.speed2",depress=False) 
                                row = col.row(align=True)
                                if bpy.context.scene.use_preview_range == True:
                                    row.operator("wm.range",icon='TIME',depress=True) 
                                else:
                                    row.operator("wm.range",icon='TIME',depress=False)   
                                        
                                row = col.row(align=True)  
                                row.operator("wm.unloop") 
                                
                                filepath = ob.data.filepath
                                name= ob.data.name
                                frame_duration=bpy.data.images[name].frame_duration
                                sizeX=bpy.data.images[name].size[0]
                                sizeY=bpy.data.images[name].size[1]
                                frame_current=bpy.data.objects[ob.name].image_user.frame_current    
                                row = col.row(align=True)                           
                                row.label(text= "信息:", icon= 'INFO')   
                                row = col.row(align=True) 
                                row.label(text="") 
                                row.scale_x = 3                             
                                row.label(text= "文件格式: "+str(word))     
                                row = col.row(align=True)
                                row.label(text="") 
                                row.scale_x = 3
                                row.label(text="帧: "+str(frame_duration))                                                                                        
                                row = col.row(align=True)
                                row.label(text="") 
                                row.scale_x = 3
                                row.label(text="大小: "+str(sizeX)+"x"+str(sizeY))                                          
                                row = col.row(align=True)
                                row.label(text="") 
                                row.scale_x = 3                          
                                row.label(text="当前帧: "+str(frame_current))                                           


                                
                    if ob.type != 'EMPTY':
                        pass

                col.separator() 
                row = col.row(align=True) 
                row.operator("wm.deletemovies",icon='ERROR') 

#   SETTINGS 
 
class BRSETTINGS_PT_PANEL(blendrefMAINPanel,Panel):
    bl_parent_id = "BLENDREF_PT_PANEL"
    bl_label = "设置"
    bl_options = {"DEFAULT_CLOSED"}
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        bref_prop = scene.bref_prop 
        
        box = layout.box()
        split = box.split()
        col = split.column(align=True) 
#        row = col.row(align=True)        
#        row.label(text="Settings :")
        if bpy.context.area.spaces[0].region_3d.lock_rotation == True: 
            row = col.row(align=True)  
            row.operator("wm.refresh") 
            #row.operator("wm.restore",icon='SCREEN_BACK') 
        row = col.row(align=True) 
        row.label(text="Location :")       
        row.prop(bref_prop,"xloc")      
        row.prop(bref_prop,"yloc")  
        row = col.row(align=True)
        row.prop(bref_prop, "my_col", text="颜色")  
        if bpy.context.area.spaces[0].region_3d.lock_rotation == True:
            row.operator("wm.update")


##### TEST BUTTON        
#        col.separator() 
#        col.separator() 
#        col.separator() 
#        col.alert = True 
#        row = col.row(align=True)
#        row.operator("wm.testbutton")
#        row.scale_y=4 

     
# ------------------------------------------------------------------------
#    Registration
# ------------------------------------------------------------------------

classes = (
    BLENDREF_prop,
    BLENDREF_PT_PANEL,
    BRIMAGE_PT_PANEL,
    BRMOVIE_PT_PANEL,
    BRSETTINGS_PT_PANEL,
    TEST_button,
    APPEND_button,
    REFRESH_button,
    DELETEIMAGES_button,
    DELETEMOVIES_button,
    UPDATE_button,
    FLIPX_button,
    FLIPXALL_button,
    FLIPY_button,
    UNFLIP_button,
    ImportImageOperator,
    ImportImageSeqOperator,
    ImportVideoOperator,
    FREEZE_button,
    UNFREEZE_button,
    CLONE_button,
    A_button,
    B_button,
    UNLOOP_button,
    COLONNES_button,
    COLONNES2_button,
    SPEED0_button,
    SPEED1_button,
    SPEED2_button,
    RANGE_button,
    OFFSET_button,
    RESTORE_button,
    DRAG_button,
    ENABLE_button,
    DISABLE_button,
    SAVETO_button,
    SCREENSHOT_button,
    SCREENSHOT2_button,
    LCK_button,
    UNLCK_button,
    TXT_button,
)
        
def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    bpy.types.Scene.bref_prop = PointerProperty(type=BLENDREF_prop)

  


def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
    del bpy.types.Scene.bref_prop

if __name__ == "__main__":
    register()
